from argparse import ArgumentParser
import zmq
import logging
from czmq import Zactor, zactor_fn, create_string_buffer
import os
import os.path
from pyzyre.utils import resolve_gossip, resolve_endpoint
import names
from pprint import pprint
from ._client_task import task as client_task
import sys
from zmq.eventloop import ioloop
from time import sleep
from pyzyre.constants import GOSSIP_PORT, SERVICE_PORT, ZYRE_GROUP, LOG_FORMAT, PYVERSION, GOSSIP_CONNECT, ENDPOINT

NODE_NAME = os.getenv('ZYRE_NODE_NAME')

logger = logging.getLogger(__name__)


class DefaultHandler(object):
    def on_shout(self, client, group, peer, address, message):
        pass

    def on_whisper(self, client, peer, message):
        pass

    def on_enter(self, client, peer):
        pass

    def on_join(self, client, peer, group):
        pass

    def on_leave(self, client, peer, group):
        pass

    def on_exit(self, client, peer):
        pass


class Client(object):

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        if self.actor:
            self.stop_zyre()
        return self

    def __init__(self, handler=DefaultHandler(), **kwargs):

        # disable CZMQ from capturing SIGINT
        os.environ['ZSYS_SIGHANDLER'] = 'false'

        self.handler = handler

        self.group = kwargs.get('group', ZYRE_GROUP)
        self._groups = self.group.split(',')
        self.group = '|'.join(self.group.split(','))
        self.interface = kwargs.get('interface') or '*'

        self.parent_loop = kwargs.get('loop')
        self.gossip_bind = kwargs.get('gossip_bind')
        self.beacon = kwargs.get('beacon')
        self.gossip_connect = kwargs.get('gossip_connect', GOSSIP_CONNECT)
        self.endpoint = kwargs.get('endpoint', ENDPOINT)

        self.name = kwargs.get('name', NODE_NAME)
        if not self.name:
            self.name = '{}_{}'.format(names.get_first_name().lower(), names.get_last_name().lower())

        self.actor = None
        self.task = zactor_fn(client_task)
        self.verbose = kwargs.get('verbose')

        if self.gossip_bind:
            self.beacon = False

        elif self.gossip_connect:
            self.beacon = False

        else:
            self.beacon = True

        self._init_zyre()

    def _init_beacon(self):
        logger.debug(self.interface)
        os.environ["ZSYS_INTERFACE"] = self.interface

    def _init_gossip_bind(self):
        # is gossip_bind an interface?
        if len(self.gossip_bind) <= 5:
            # we need this to resolve the endpoint
            self.interface = self.gossip_bind
            if not self.endpoint:
                self.endpoint = resolve_endpoint(SERVICE_PORT, interface=self.interface)

        self.gossip_bind = resolve_gossip(GOSSIP_PORT, self.gossip_bind)
        logger.debug('gossip-bind: %s' % self.gossip_bind)

        if not self.endpoint:
            if self.interface:
                self.endpoint = resolve_endpoint(SERVICE_PORT, interface=self.interface)
            else:
                raise RuntimeError('A local interface must be specified')

    def _init_gossip_connect(self):
        try:
            logger.info('resolving gossip-connect: {}'.format(self.gossip_connect))
            self.gossip_connect = resolve_gossip(GOSSIP_PORT, self.gossip_connect)
            logger.debug('gossip-connect: %s' % self.gossip_connect)

        except RuntimeError as e:
            logger.error(e)
            logger.debug('falling back to beacon mode..')
            self.beacon = 1
            self.gossip_connect = None

        if not self.endpoint:
            if self.interface:
                self.endpoint = resolve_endpoint(SERVICE_PORT, interface=self.interface)
            else:
                raise RuntimeError('A local interface must be specified')

    def _init_zyre(self):
        # setup czmq/zyre
        # disable CZMQ from capturing SIGINT
        os.environ['ZSYS_SIGHANDLER'] = 'false'

        if self.gossip_bind:
            self._init_gossip_bind()

        if self.gossip_connect:
            self._init_gossip_connect()

        if self.beacon:
            self._init_beacon()

        actor_args = [
            'group=%s' % self.group,
            'name=%s' % self.name,
        ]

        if self.verbose:
            actor_args.append('verbose=1')

        if self.gossip_bind:
            actor_args.append('gossip_bind=%s' % self.gossip_bind)

        elif self.gossip_connect:
            actor_args.append('gossip_connect=%s' % self.gossip_connect)

        else:
            actor_args.append('beacon=1')

        if self.endpoint:
            actor_args.append('endpoint=%s' % self.endpoint)

        actor_args = ','.join(actor_args)
        self.actor_args = create_string_buffer(actor_args)

        self.actor = None
        self._actor = None

    def start_zyre(self):
        self._actor = Zactor(self.task, self.actor_args)
        self.actor = zmq.Socket(shadow=self._actor.resolve(self._actor).value)

    def stop_zyre(self):
        self.actor.send_multipart(['$$STOP'.encode('utf-8')])
        m = self.actor.recv_multipart()
        sleep(0.01)
        del self._actor

    def join(self, group):
        logger.debug('sending join')
        self.actor.send_multipart(['JOIN', group.encode('utf-8')])

    def shout(self, group, message):
        self.actor.send_multipart(['SHOUT', group.encode('utf-8'), message.encode('utf-8')])

    def whisper(self, message, address):
        logger.debug('sending whisper to %s' % address)
        self.actor.send_multipart(['WHISPER', address, message.encode('utf-8')])
        logger.debug('message sent via whisper')

    # deprecate
    def send_message(self, message, group=None, address=None):
        if isinstance(message, str) and PYVERSION == 2:
            message = unicode(message, 'utf-8')

        if address:
            logger.debug('sending whisper to %s' % address)
            self.actor.send_multipart(['WHISPER', address, message.encode('utf-8')])
            logger.debug('message sent via whisper')
        else:
            if not group:
                group = self._groups[0]
            self.actor.send_multipart(['SHOUT', group.encode('utf-8'), message.encode('utf-8')])

    def handle_message(self, s, e):
        m = s.recv_multipart()

        m_type = m.pop(0)

        if m_type == 'SHOUT':
            group, peer, address, message = m
            self.handler.on_shout(self, group, peer, address, message)

        elif m_type == 'ENTER':
            self.handler.on_enter(self, peer=m)

        elif m_type == 'WHISPER':
            peer, message = m
            self.handler.on_whisper(self, peer, message)

        elif m_type == 'EXIT':
            peer, peers_remining = m
            logger.debug(peers_remining)
            if self.gossip_connect and peers_remining == '0':
                self.parent_loop.remove_handler(self.actor)
                self.stop_zyre()
                self.start_zyre()
                self.parent_loop.add_handler(self.actor, self.handle_message, zmq.POLLIN)
            self.handler.on_exit(self, peer)

        elif m_type == 'JOIN':
            peer, group = m
            self.handler.on_join(self, peer, group)

        elif m_type == 'LEAVE':
            peer, group = m
            self.handler.on_leave(self, peer, group)

        else:
            logger.warn("unhandled m_type {} rest of message is {}".format(m_type, m))


def main():
    p = ArgumentParser()

    endpoint = resolve_endpoint(SERVICE_PORT)

    p.add_argument('--gossip-bind', help='bind gossip endpoint on this node')
    p.add_argument('--gossip-connect')
    p.add_argument('-i', '--interface', help='specify zsys_interface for beacon')
    p.add_argument('-l', '--endpoint', help='specify ip listening endpoint [default %(default)s]', default=endpoint)
    p.add_argument('-d', '--debug', help='enable debugging', action='store_true')

    p.add_argument('--group', default=ZYRE_GROUP)

    args = p.parse_args()

    loglevel = logging.INFO
    verbose = False
    if args.debug:
        loglevel = logging.DEBUG
        verbose = '1'

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)
    logging.propagate = False

    ioloop.install()
    loop = ioloop.IOLoop.instance()

    client = Client(
        group=args.group,
        loop=loop,
        gossip_bind=args.gossip_bind,
        gossip_connect=args.gossip_connect,
        endpoint=args.endpoint,
        verbose=verbose,
        interface=args.interface,
    )

    def on_stdin(s, e):
        content = s.readline().rstrip()

        if content.startswith('CLIENT:'):
            address, message = content.split(' - ')
            address = address.split(':')[1]
            client.whisper(message, address)
        else:
            client.shout(args.group, content.encode('utf-8'))

    client.start_zyre()

    loop.add_handler(client.actor, client.handle_message, zmq.POLLIN)
    loop.add_handler(sys.stdin, on_stdin, ioloop.IOLoop.READ)

    try:
        loop.start()
    except KeyboardInterrupt:
        logger.info('SIGINT Received')

    logger.info('shutting down..')

    client.stop_zyre()

if __name__ == '__main__':
    main()



################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################

from __future__ import print_function
import os, sys
from ctypes import *
from ctypes.util import find_library
import czmq

# load libc to access free, etc.
libcpath = find_library("c")
if not libcpath:
    raise ImportError("Unable to find libc")
libc = cdll.LoadLibrary(libcpath)
libc.free.argtypes = [c_void_p]
libc.free.restype = None

def return_fresh_string(char_p):
    s = string_at(char_p)
    libc.free(char_p)
    return s

# zyre
lib = None
try:
    # check to see if the shared object was embedded locally, attempt to load it
    # if not, try to load it using the default system paths...
    # we need to use os.chdir instead of trying to modify $LD_LIBRARY_PATH and reloading the interpreter
    t = os.getcwd()
    p = os.path.join(os.path.dirname(__file__), '..')  # find the path to our $project_ctypes.py
    os.chdir(p)  # change directories briefly

    from zyre import libzyre                        # attempt to import the shared lib if it exists
    lib = CDLL(libzyre.__file__)             # if it exists try to load the shared lib
    os.chdir(t)  # switch back to orig dir
except ImportError:
    pass

if not lib:
    try:
        # If LD_LIBRARY_PATH or your OSs equivalent is set, this is the only way to
        # load the library.  If we use find_library below, we get the wrong result.
        if os.name == 'posix':
            if sys.platform == 'darwin':
                lib = cdll.LoadLibrary('libzyre.2.dylib')
            else:
                lib = cdll.LoadLibrary("libzyre.so.2")
        elif os.name == 'nt':
            lib = cdll.LoadLibrary('libzyre.dll')
    except OSError:
        libpath = find_library("zyre")
        if not libpath:
            raise ImportError("Unable to find libzyre")
        lib = cdll.LoadLibrary(libpath)

class zyre_t(Structure):
    pass # Empty - only for type checking
zyre_p = POINTER(zyre_t)

class zyre_event_t(Structure):
    pass # Empty - only for type checking
zyre_event_p = POINTER(zyre_event_t)


# zyre
lib.zyre_new.restype = zyre_p
lib.zyre_new.argtypes = [c_char_p]
lib.zyre_destroy.restype = None
lib.zyre_destroy.argtypes = [POINTER(zyre_p)]
lib.zyre_uuid.restype = c_char_p
lib.zyre_uuid.argtypes = [zyre_p]
lib.zyre_name.restype = c_char_p
lib.zyre_name.argtypes = [zyre_p]
lib.zyre_set_header.restype = None
lib.zyre_set_header.argtypes = [zyre_p, c_char_p, c_char_p]
lib.zyre_set_verbose.restype = None
lib.zyre_set_verbose.argtypes = [zyre_p]
lib.zyre_set_port.restype = None
lib.zyre_set_port.argtypes = [zyre_p, c_int]
lib.zyre_set_evasive_timeout.restype = None
lib.zyre_set_evasive_timeout.argtypes = [zyre_p, c_int]
lib.zyre_set_expired_timeout.restype = None
lib.zyre_set_expired_timeout.argtypes = [zyre_p, c_int]
lib.zyre_set_interval.restype = None
lib.zyre_set_interval.argtypes = [zyre_p, c_size_t]
lib.zyre_set_interface.restype = None
lib.zyre_set_interface.argtypes = [zyre_p, c_char_p]
lib.zyre_set_endpoint.restype = c_int
lib.zyre_set_endpoint.argtypes = [zyre_p, c_char_p]
lib.zyre_gossip_bind.restype = None
lib.zyre_gossip_bind.argtypes = [zyre_p, c_char_p]
lib.zyre_gossip_connect.restype = None
lib.zyre_gossip_connect.argtypes = [zyre_p, c_char_p]
lib.zyre_start.restype = c_int
lib.zyre_start.argtypes = [zyre_p]
lib.zyre_stop.restype = None
lib.zyre_stop.argtypes = [zyre_p]
lib.zyre_join.restype = c_int
lib.zyre_join.argtypes = [zyre_p, c_char_p]
lib.zyre_leave.restype = c_int
lib.zyre_leave.argtypes = [zyre_p, c_char_p]
lib.zyre_recv.restype = czmq.zmsg_p
lib.zyre_recv.argtypes = [zyre_p]
lib.zyre_whisper.restype = c_int
lib.zyre_whisper.argtypes = [zyre_p, c_char_p, POINTER(czmq.zmsg_p)]
lib.zyre_shout.restype = c_int
lib.zyre_shout.argtypes = [zyre_p, c_char_p, POINTER(czmq.zmsg_p)]
lib.zyre_whispers.restype = c_int
lib.zyre_whispers.argtypes = [zyre_p, c_char_p, c_char_p]
lib.zyre_shouts.restype = c_int
lib.zyre_shouts.argtypes = [zyre_p, c_char_p, c_char_p]
lib.zyre_peers.restype = czmq.zlist_p
lib.zyre_peers.argtypes = [zyre_p]
lib.zyre_peers_by_group.restype = czmq.zlist_p
lib.zyre_peers_by_group.argtypes = [zyre_p, c_char_p]
lib.zyre_own_groups.restype = czmq.zlist_p
lib.zyre_own_groups.argtypes = [zyre_p]
lib.zyre_peer_groups.restype = czmq.zlist_p
lib.zyre_peer_groups.argtypes = [zyre_p]
lib.zyre_peer_address.restype = POINTER(c_char)
lib.zyre_peer_address.argtypes = [zyre_p, c_char_p]
lib.zyre_peer_header_value.restype = POINTER(c_char)
lib.zyre_peer_header_value.argtypes = [zyre_p, c_char_p, c_char_p]
lib.zyre_socket.restype = czmq.zsock_p
lib.zyre_socket.argtypes = [zyre_p]
lib.zyre_print.restype = None
lib.zyre_print.argtypes = [zyre_p]
lib.zyre_version.restype = c_long
lib.zyre_version.argtypes = []
lib.zyre_test.restype = None
lib.zyre_test.argtypes = [c_bool]

class Zyre(object):
    """
    An open-source framework for proximity-based P2P apps
    """

    allow_destruct = False
    def __init__(self, *args):
        """
        Constructor, creates a new Zyre node. Note that until you start the
node it is silent and invisible to other nodes on the network.
The node name is provided to other nodes during discovery. If you
specify NULL, Zyre generates a randomized node name from the UUID.
        """
        if len(args) == 2 and type(args[0]) is c_void_p and isinstance(args[1], bool):
            self._as_parameter_ = cast(args[0], zyre_p) # Conversion from raw type to binding
            self.allow_destruct = args[1] # This is a 'fresh' value, owned by us
        elif len(args) == 2 and type(args[0]) is zyre_p and isinstance(args[1], bool):
            self._as_parameter_ = args[0] # Conversion from raw type to binding
            self.allow_destruct = args[1] # This is a 'fresh' value, owned by us
        else:
            assert(len(args) == 1)
            self._as_parameter_ = lib.zyre_new(args[0]) # Creation of new raw type
            self.allow_destruct = True

    def __del__(self):
        """
        Destructor, destroys a Zyre node. When you destroy a node, any
messages it is sending or receiving will be discarded.
        """
        if self.allow_destruct:
            lib.zyre_destroy(byref(self._as_parameter_))

    def __eq__(self, other):
        if type(other) == type(self):
            return other.c_address() == self.c_address()
        elif type(other) == c_void_p:
            return other.value == self.c_address()

    def c_address(self):
        """
        Return the address of the object pointer in c.  Useful for comparison.
        """
        return addressof(self._as_parameter_.contents)

    def __bool__(self):
        "Determine whether the object is valid by converting to boolean" # Python 3
        return self._as_parameter_.__bool__()

    def __nonzero__(self):
        "Determine whether the object is valid by converting to boolean" # Python 2
        return self._as_parameter_.__nonzero__()

    def uuid(self):
        """
        Return our node UUID string, after successful initialization
        """
        return lib.zyre_uuid(self._as_parameter_)

    def name(self):
        """
        Return our node name, after successful initialization
        """
        return lib.zyre_name(self._as_parameter_)

    def set_header(self, name, format, *args):
        """
        Set node header; these are provided to other nodes during discovery
and come in each ENTER message.
        """
        return lib.zyre_set_header(self._as_parameter_, name, format, *args)

    def set_verbose(self):
        """
        Set verbose mode; this tells the node to log all traffic as well as
all major events.
        """
        return lib.zyre_set_verbose(self._as_parameter_)

    def set_port(self, port_nbr):
        """
        Set UDP beacon discovery port; defaults to 5670, this call overrides
that so you can create independent clusters on the same network, for
e.g. development vs. production. Has no effect after zyre_start().
        """
        return lib.zyre_set_port(self._as_parameter_, port_nbr)

    def set_evasive_timeout(self, interval):
        """
        Set the peer evasiveness timeout, in milliseconds. Default is 5000.
This can be tuned in order to deal with expected network conditions
and the response time expected by the application. This is tied to
the beacon interval and rate of messages received.
        """
        return lib.zyre_set_evasive_timeout(self._as_parameter_, interval)

    def set_expired_timeout(self, interval):
        """
        Set the peer expiration timeout, in milliseconds. Default is 30000.
This can be tuned in order to deal with expected network conditions
and the response time expected by the application. This is tied to
the beacon interval and rate of messages received.
        """
        return lib.zyre_set_expired_timeout(self._as_parameter_, interval)

    def set_interval(self, interval):
        """
        Set UDP beacon discovery interval, in milliseconds. Default is instant
beacon exploration followed by pinging every 1,000 msecs.
        """
        return lib.zyre_set_interval(self._as_parameter_, interval)

    def set_interface(self, value):
        """
        Set network interface for UDP beacons. If you do not set this, CZMQ will
choose an interface for you. On boxes with several interfaces you should
specify which one you want to use, or strange things can happen.
        """
        return lib.zyre_set_interface(self._as_parameter_, value)

    def set_endpoint(self, format, *args):
        """
        By default, Zyre binds to an ephemeral TCP port and broadcasts the local
host name using UDP beaconing. When you call this method, Zyre will use
gossip discovery instead of UDP beaconing. You MUST set-up the gossip
service separately using zyre_gossip_bind() and _connect(). Note that the
endpoint MUST be valid for both bind and connect operations. You can use
inproc://, ipc://, or tcp:// transports (for tcp://, use an IP address
that is meaningful to remote as well as local nodes). Returns 0 if
the bind was successful, else -1.
        """
        return lib.zyre_set_endpoint(self._as_parameter_, format, *args)

    def gossip_bind(self, format, *args):
        """
        Set-up gossip discovery of other nodes. At least one node in the cluster
must bind to a well-known gossip endpoint, so other nodes can connect to
it. Note that gossip endpoints are completely distinct from Zyre node
endpoints, and should not overlap (they can use the same transport).
        """
        return lib.zyre_gossip_bind(self._as_parameter_, format, *args)

    def gossip_connect(self, format, *args):
        """
        Set-up gossip discovery of other nodes. A node may connect to multiple
other nodes, for redundancy paths. For details of the gossip network
design, see the CZMQ zgossip class.
        """
        return lib.zyre_gossip_connect(self._as_parameter_, format, *args)

    def start(self):
        """
        Start node, after setting header values. When you start a node it
begins discovery and connection. Returns 0 if OK, -1 if it wasn't
possible to start the node.
        """
        return lib.zyre_start(self._as_parameter_)

    def stop(self):
        """
        Stop node; this signals to other peers that this node will go away.
This is polite; however you can also just destroy the node without
stopping it.
        """
        return lib.zyre_stop(self._as_parameter_)

    def join(self, group):
        """
        Join a named group; after joining a group you can send messages to
the group and all Zyre nodes in that group will receive them.
        """
        return lib.zyre_join(self._as_parameter_, group)

    def leave(self, group):
        """
        Leave a group
        """
        return lib.zyre_leave(self._as_parameter_, group)

    def recv(self):
        """
        Receive next message from network; the message may be a control
message (ENTER, EXIT, JOIN, LEAVE) or data (WHISPER, SHOUT).
Returns zmsg_t object, or NULL if interrupted
        """
        return czmq.Zmsg(lib.zyre_recv(self._as_parameter_), True)

    def whisper(self, peer, msg_p):
        """
        Send message to single peer, specified as a UUID string
Destroys message after sending
        """
        return lib.zyre_whisper(self._as_parameter_, peer, byref(czmq.zmsg_p.from_param(msg_p)))

    def shout(self, group, msg_p):
        """
        Send message to a named group
Destroys message after sending
        """
        return lib.zyre_shout(self._as_parameter_, group, byref(czmq.zmsg_p.from_param(msg_p)))

    def whispers(self, peer, format, *args):
        """
        Send formatted string to a single peer specified as UUID string
        """
        return lib.zyre_whispers(self._as_parameter_, peer, format, *args)

    def shouts(self, group, format, *args):
        """
        Send formatted string to a named group
        """
        return lib.zyre_shouts(self._as_parameter_, group, format, *args)

    def peers(self):
        """
        Return zlist of current peer ids.
        """
        return czmq.Zlist(lib.zyre_peers(self._as_parameter_), True)

    def peers_by_group(self, name):
        """
        Return zlist of current peers of this group.
        """
        return czmq.Zlist(lib.zyre_peers_by_group(self._as_parameter_, name), True)

    def own_groups(self):
        """
        Return zlist of currently joined groups.
        """
        return czmq.Zlist(lib.zyre_own_groups(self._as_parameter_), True)

    def peer_groups(self):
        """
        Return zlist of groups known through connected peers.
        """
        return czmq.Zlist(lib.zyre_peer_groups(self._as_parameter_), True)

    def peer_address(self, peer):
        """
        Return the endpoint of a connected peer.
        """
        return return_fresh_string(lib.zyre_peer_address(self._as_parameter_, peer))

    def peer_header_value(self, peer, name):
        """
        Return the value of a header of a conected peer.
Returns null if peer or key doesn't exits.
        """
        return return_fresh_string(lib.zyre_peer_header_value(self._as_parameter_, peer, name))

    def socket(self):
        """
        Return socket for talking to the Zyre node, for polling
        """
        return czmq.Zsock(lib.zyre_socket(self._as_parameter_), False)

    def print(self):
        """
        Print zyre node information to stdout
        """
        return lib.zyre_print(self._as_parameter_)

    @staticmethod
    def version():
        """
        Return the Zyre version for run-time API detection; returns
major * 10000 + minor * 100 + patch, as a single integer.
        """
        return lib.zyre_version()

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        return lib.zyre_test(verbose)


# zyre event
lib.zyre_event_new.restype = zyre_event_p
lib.zyre_event_new.argtypes = [zyre_p]
lib.zyre_event_destroy.restype = None
lib.zyre_event_destroy.argtypes = [POINTER(zyre_event_p)]
lib.zyre_event_type.restype = c_char_p
lib.zyre_event_type.argtypes = [zyre_event_p]
lib.zyre_event_peer_uuid.restype = c_char_p
lib.zyre_event_peer_uuid.argtypes = [zyre_event_p]
lib.zyre_event_peer_name.restype = c_char_p
lib.zyre_event_peer_name.argtypes = [zyre_event_p]
lib.zyre_event_peer_addr.restype = c_char_p
lib.zyre_event_peer_addr.argtypes = [zyre_event_p]
lib.zyre_event_headers.restype = czmq.zhash_p
lib.zyre_event_headers.argtypes = [zyre_event_p]
lib.zyre_event_header.restype = c_char_p
lib.zyre_event_header.argtypes = [zyre_event_p, c_char_p]
lib.zyre_event_group.restype = c_char_p
lib.zyre_event_group.argtypes = [zyre_event_p]
lib.zyre_event_msg.restype = czmq.zmsg_p
lib.zyre_event_msg.argtypes = [zyre_event_p]
lib.zyre_event_get_msg.restype = czmq.zmsg_p
lib.zyre_event_get_msg.argtypes = [zyre_event_p]
lib.zyre_event_print.restype = None
lib.zyre_event_print.argtypes = [zyre_event_p]
lib.zyre_event_test.restype = None
lib.zyre_event_test.argtypes = [c_bool]

class ZyreEvent(object):
    """
    Parsing Zyre messages
    """

    allow_destruct = False
    def __init__(self, *args):
        """
        Constructor: receive an event from the zyre node, wraps zyre_recv.
The event may be a control message (ENTER, EXIT, JOIN, LEAVE) or
data (WHISPER, SHOUT).
        """
        if len(args) == 2 and type(args[0]) is c_void_p and isinstance(args[1], bool):
            self._as_parameter_ = cast(args[0], zyre_event_p) # Conversion from raw type to binding
            self.allow_destruct = args[1] # This is a 'fresh' value, owned by us
        elif len(args) == 2 and type(args[0]) is zyre_event_p and isinstance(args[1], bool):
            self._as_parameter_ = args[0] # Conversion from raw type to binding
            self.allow_destruct = args[1] # This is a 'fresh' value, owned by us
        else:
            assert(len(args) == 1)
            self._as_parameter_ = lib.zyre_event_new(args[0]) # Creation of new raw type
            self.allow_destruct = True

    def __del__(self):
        """
        Destructor; destroys an event instance
        """
        if self.allow_destruct:
            lib.zyre_event_destroy(byref(self._as_parameter_))

    def __eq__(self, other):
        if type(other) == type(self):
            return other.c_address() == self.c_address()
        elif type(other) == c_void_p:
            return other.value == self.c_address()

    def c_address(self):
        """
        Return the address of the object pointer in c.  Useful for comparison.
        """
        return addressof(self._as_parameter_.contents)

    def __bool__(self):
        "Determine whether the object is valid by converting to boolean" # Python 3
        return self._as_parameter_.__bool__()

    def __nonzero__(self):
        "Determine whether the object is valid by converting to boolean" # Python 2
        return self._as_parameter_.__nonzero__()

    def type(self):
        """
        Returns event type, as printable uppercase string. Choices are:
"ENTER", "EXIT", "JOIN", "LEAVE", "EVASIVE", "WHISPER" and "SHOUT"
and for the local node: "STOP"
        """
        return lib.zyre_event_type(self._as_parameter_)

    def peer_uuid(self):
        """
        Return the sending peer's uuid as a string
        """
        return lib.zyre_event_peer_uuid(self._as_parameter_)

    def peer_name(self):
        """
        Return the sending peer's public name as a string
        """
        return lib.zyre_event_peer_name(self._as_parameter_)

    def peer_addr(self):
        """
        Return the sending peer's ipaddress as a string
        """
        return lib.zyre_event_peer_addr(self._as_parameter_)

    def headers(self):
        """
        Returns the event headers, or NULL if there are none
        """
        return czmq.Zhash(lib.zyre_event_headers(self._as_parameter_), False)

    def header(self, name):
        """
        Returns value of a header from the message headers
obtained by ENTER. Return NULL if no value was found.
        """
        return lib.zyre_event_header(self._as_parameter_, name)

    def group(self):
        """
        Returns the group name that a SHOUT event was sent to
        """
        return lib.zyre_event_group(self._as_parameter_)

    def msg(self):
        """
        Returns the incoming message payload; the caller can modify the
message but does not own it and should not destroy it.
        """
        return czmq.Zmsg(lib.zyre_event_msg(self._as_parameter_), False)

    def get_msg(self):
        """
        Returns the incoming message payload, and pass ownership to the
caller. The caller must destroy the message when finished with it.
After called on the given event, further calls will return NULL.
        """
        return czmq.Zmsg(lib.zyre_event_get_msg(self._as_parameter_), True)

    def print(self):
        """
        Print event to zsys log
        """
        return lib.zyre_event_print(self._as_parameter_)

    @staticmethod
    def test(verbose):
        """
        Self test of this class.
        """
        return lib.zyre_event_test(verbose)

################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################



import sys
import zmq

topic = sys.argv[1]

# Socket to talk to server
context = zmq.Context()
control = context.socket(zmq.PUSH)
control.connect("tcp://localhost:%s" % 5002)
control.send_multipart(["SUB", topic])

while True:
    msg = sys.stdin.readline().rstrip()
    if not msg:
        break
    control.send_multipart(["PUB", topic, msg])



from distutils.ccompiler import get_default_compiler, new_compiler
from distutils.extension import Extension
from distutils.command.build_ext import build_ext
from distutils.command.sdist import sdist
import shutil
from glob import glob
from os.path import basename, join as pjoin
from subprocess import Popen, PIPE
from ctypes import *

from .msg import *
from .fetch import fetch_libzyre

HERE = os.path.dirname(__file__)
ROOT = os.path.dirname(HERE)


def stage_platform_header(zmqroot):
    platform_h = pjoin(zmqroot, 'src', 'platform.h')

    if os.path.exists(platform_h):
        info("already have platform.h")
        return

    if os.name == 'nt':
        # stage msvc platform header
        platform_dir = pjoin(zmqroot, 'builds', 'msvc')
    else:
        if not os.path.exists(pjoin(zmqroot, 'configure')):
            info('attempting bash autogen.sh')
            p = Popen('./autogen.sh', cwd=zmqroot, shell=True, stdout=PIPE, stderr=PIPE)
            o, e = p.communicate()
            if p.returncode:
                raise RuntimeError('Failed to run autoconf...')

        configure_cmd = [
            'libzmq_CFLAGS=-I../zeromq/include',
            'libzmq_LIBS=-L../zeromq/src',
            'czmq_CFLAGS=-I../czmq/include',
            'czmq_LIBS=-L../czmq/src',
            './configure'

        ]
        info("attempting ./configure to generate platform.h with: {}".format(configure_cmd))
        p = Popen(' '.join(configure_cmd), cwd=zmqroot, shell=True, stdout=PIPE, stderr=PIPE)

        o, e = p.communicate()
        if p.returncode:
            warn("failed to configure libzyre:\n%s" % e)

            if sys.platform == 'darwin':
                platform_dir = pjoin(HERE, 'include_darwin')
            elif sys.platform.startswith('freebsd'):
                platform_dir = pjoin(HERE, 'include_freebsd')
            elif sys.platform.startswith('linux-armv'):
                platform_dir = pjoin(HERE, 'include_linux-armv')
            else:
                platform_dir = pjoin(HERE, 'include_linux')
        else:
            return

    info("staging platform.h from: %s" % platform_dir)
    shutil.copy(pjoin(platform_dir, 'platform.h'), platform_h)


class ConfigureSDist(sdist):
    def run(self):
        bundledir = "bundled"

        line()

        if not os.path.exists(bundledir):
            os.makedirs(bundledir)

        fetch_libzyre(bundledir)

        bundledincludedir = 'zyre'

        if not os.path.exists(bundledincludedir):
            os.makedirs(bundledincludedir)

        files = [
            pjoin(bundledir, 'zyre', 'bindings', 'python', 'zyre', '_zyre_ctypes.py'),
            pjoin(bundledir, 'zyre', 'bindings', 'python', 'zyre', '__init__.py')
        ]

        for f in files:
            shutil.copyfile(f, pjoin(bundledincludedir, basename(f)))
            #shutil.copyfile(f, pjoin(self.build_lib, bundledincludedir, basename(f)))

class Configure(build_ext):

    user_options = build_ext.user_options + [
        ('zyre=', None, "libzyre install prefix"),
        ('build-base=', 'b', "base directory for build library"),  # build_base from build
    ]

    def initialize_options(self):
        build_ext.initialize_options(self)
        self.zmq = None
        self.build_base = 'build'

    # DON'T REMOVE: distutils demands these be here even if they do nothing.
    def finalize_options(self):
        build_ext.finalize_options(self)
        self.tempdir = pjoin(self.build_temp, 'scratch')
        self.has_run = False

    @property
    def compiler_type(self):
        compiler = self.compiler
        if compiler is None:
            return get_default_compiler()
        elif isinstance(compiler, str):
            return compiler
        else:
            return compiler.compiler_type

    @property
    def cross_compiling(self):
        return self.config['bdist_egg'].get('plat-name', sys.platform) != sys.platform

    def bundle_libzyre_extension(self):
        bundledir = "bundled"
        ext_modules = self.distribution.ext_modules

        if ext_modules and any(m.name == 'zyre.libzyre' for m in ext_modules):
            # I've already been run
            return

        line()
        info("Using bundled libzyre")

        # fetch sources for libzmq extension:
        if not os.path.exists(bundledir):
            os.makedirs(bundledir)

        fetch_libzyre(bundledir)

        stage_platform_header(pjoin(bundledir, 'zyre'))

        zyre_sources = []
        for f in glob(pjoin(bundledir, 'zyre', 'src', '*.c')):
            if f not in ['bundled/zyre/src/ztester_gossip.c', 'bundled/zyre/src/zpinger.c',
                         'bundled/zyre/src/ztester_beacon.c', 'bundled/zyre/src/zyre_selftest.c',
                         'bundled/zyre/src/perf_remote.c']:
                zyre_sources.append(f)

        zyre_sources.append(pjoin('buildutils', 'zyre', 'initlibzyre.c'))

        compile_args = [
            '-std=c99',
            '-Wno-strict-prototypes',
            '-Wno-unused-variable',
            '-Wno-unused-function',
        ]

        if sys.platform != 'darwin':
            compile_args.append('-Wno-unused-but-set-variable')

        libzyre = Extension(
            'zyre.libzyre',
            sources=zyre_sources,
            include_dirs=[
                pjoin(bundledir, 'zeromq', 'include'),
                pjoin(bundledir, 'czmq', 'include'),
                pjoin(bundledir, 'zyre', 'include')
            ],
            libraries=['zmq', 'czmq'],
            library_dirs=[
                pjoin(self.build_lib, 'czmq'),
                pjoin(self.build_lib, 'zyre')
            ],
            extra_compile_args=compile_args,
            # http://stackoverflow.com/a/19147134
            runtime_library_dirs=['zmq', 'czmq', 'zyre', '.'],
        )

        # register the extension:
        self.distribution.ext_modules.insert(2, libzyre)

        libzyre.include_dirs.append(bundledir)

        libzyre.define_macros.append(('ENABLE_DRAFTS', 1))

        cc = new_compiler(compiler=self.compiler_type)
        cc.output_dir = self.build_temp

        bundledincludedir = 'zyre'

        if not os.path.exists(bundledincludedir):
            os.makedirs(bundledincludedir)

        if not os.path.exists(pjoin(self.build_lib, bundledincludedir)):
            os.makedirs(pjoin(self.build_lib, bundledincludedir))

        files = [
            pjoin(bundledir, 'zyre', 'bindings', 'python', 'zyre', '_zyre_ctypes.py'),
            pjoin(bundledir, 'zyre', 'bindings', 'python', 'zyre', '__init__.py')
        ]

        for f in files:
            shutil.copyfile(f, pjoin(bundledincludedir, basename(f)))
            shutil.copyfile(f, pjoin(self.build_lib, bundledincludedir, basename(f)))

    def run(self):

        self.bundle_libzyre_extension()


from argparse import ArgumentParser
import zmq
import logging
from pyzyre.utils import resolve_endpoint
from zmq.eventloop import ioloop
from pyzyre.constants import SERVICE_PORT, ZYRE_GROUP, LOG_FORMAT
from pyzyre.client import Client, DefaultHandler


logger = logging.getLogger(__name__)


class GatewayHandler(DefaultHandler):
    def __init__(self, pub):
        self.pub = pub

    def on_shout(self, client, group, peer, address, message):
        self.pub.send_multipart([group, message])


# see examples/pub.py and sub.py
def main():
    p = ArgumentParser()

    endpoint = resolve_endpoint(SERVICE_PORT)

    p.add_argument('--gossip-bind', help='bind gossip endpoint on this node')
    p.add_argument('--gossip-connect')
    p.add_argument('-i', '--interface', help='specify zsys_interface for beacon')
    p.add_argument('-l', '--endpoint', help='specify ip listening endpoint [default %(default)s]', default=endpoint)
    p.add_argument('-d', '--debug', help='enable debugging', action='store_true')
    p.add_argument('-p', '--pub', help='endpoint to bind for PUB socket [default %(default)s]', default="tcp://*:5001")
    p.add_argument('-u', '--pull', help='endpoint to bind for PULL socket [default %(default)s]', default="tcp://*:5002")

    p.add_argument('--group', default=ZYRE_GROUP)

    args = p.parse_args()

    loglevel = logging.INFO
    verbose = False
    if args.debug:
        loglevel = logging.DEBUG
        verbose = '1'

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)
    logging.propagate = False

    ioloop.install()
    loop = ioloop.IOLoop.instance()

    context = zmq.Context()
    pub = context.socket(zmq.PUB)
    if not args.pub.startswith(('tcp://', 'udp://', 'ipc://')):
        args.pub = 'tcp://%s' % args.pub

    logger.info('listing for PUB on %s' % args.pub)
    pub.bind(args.pub)

    pull = context.socket(zmq.PULL)
    if not args.pull.startswith(('tcp://', 'udp://', 'ipc://')):
        args.pull = 'tcp://%s' % args.pull

    logger.info('listing for PULL on %s' % args.pull)
    pull.bind(args.pull)

    client = Client(
        handler=GatewayHandler(pub),
        group=args.group,
        loop=loop,
        gossip_bind=args.gossip_bind,
        gossip_connect=args.gossip_connect,
        endpoint=args.endpoint,
        verbose=verbose,
        interface=args.interface,
    )

    def handle_pull(*args):
        req = pull.recv_multipart()
        cmd = req[0]
        args = req[1:]
        if cmd == 'PUB':
            client.shout(*args)
            pub.send_multipart(args)
        elif cmd == 'SUB':
            client.join(*args)

    client.start_zyre()
    loop.add_handler(client.actor, client.handle_message, zmq.POLLIN)
    loop.add_handler(pull, handle_pull, zmq.POLLIN)

    terminated = False
    while not terminated:
        try:
            logger.info('starting loop...')
            loop.start()
        except KeyboardInterrupt:
            logger.info('SIGINT Received')

            terminated = True
        except Exception as e:
            terminated = True
            logger.error(e)

    logger.info('shutting down..')

    client.stop_zyre()

if __name__ == '__main__':
    main()



from __future__ import with_statement, print_function

import os
import shutil
import sys

from distutils.ccompiler import get_default_compiler
from distutils.ccompiler import new_compiler
from distutils.extension import Extension
from distutils.command.build_ext import build_ext

from glob import glob
from os.path import basename, join as pjoin
from os.path import basename, join as pjoin
from subprocess import Popen, PIPE

from .bundle import bundled_version, fetch_libzmq, localpath
from .msg import fatal, warn, info, line


libzmq_name = 'libzmq'
pypy = 'PyPy' in sys.version

# reference points for zmq compatibility

min_legacy_zmq = (2, 1, 4)
min_good_zmq = (3, 2)
target_zmq = bundled_version
dev_zmq = (target_zmq[0], target_zmq[1] + 1, 0)

HERE = os.path.dirname(__file__)
ROOT = os.path.dirname(HERE)

# set dylib ext:
if sys.platform.startswith('win'):
    lib_ext = '.dll'
elif sys.platform == 'darwin':
    lib_ext = '.dylib'
else:
    lib_ext = '.so'


def stage_platform_header(zmqroot):
    platform_h = pjoin(zmqroot, 'src', 'platform.hpp')

    if os.path.exists(platform_h):
        info("already have platform.hpp")
        return

    if os.name == 'nt':
        # stage msvc platform header
        platform_dir = pjoin(zmqroot, 'builds', 'msvc')
    else:
        if not os.path.exists(pjoin(zmqroot, 'configure')):
            info('attempting bash autogen.sh')
            p = Popen('./autogen.sh', cwd=zmqroot, shell=True, stdout=PIPE, stderr=PIPE)
            o, e = p.communicate()
            if p.returncode:
                raise RuntimeError('Failed to run autoconf...')

        info("attempting ./configure to generate platform.h")

        p = Popen('./configure', cwd=zmqroot, shell=True, stdout=PIPE, stderr=PIPE)

        o, e = p.communicate()
        if p.returncode:
            warn("failed to configure libczmq:\n%s" % e)

            if sys.platform == 'darwin':
                platform_dir = pjoin(HERE, 'include_darwin')
            elif sys.platform.startswith('freebsd'):
                platform_dir = pjoin(HERE, 'include_freebsd')
            elif sys.platform.startswith('linux-armv'):
                platform_dir = pjoin(HERE, 'include_linux-armv')
            else:
                platform_dir = pjoin(HERE, 'include_linux')
        else:
            return

    info("staging platform.h from: %s" % platform_dir)
    shutil.copy(pjoin(platform_dir, 'platform.h'), platform_h)


class Configure(build_ext):
    """Configure command adapted from h5py"""

    description = "Discover ZMQ version and features"

    user_options = build_ext.user_options + [
        ('zmq=', None, "libzmq install prefix"),
        ('build-base=', 'b', "base directory for build library"),  # build_base from build
        ('embed', None, '')

    ]

    def initialize_options(self):
        build_ext.initialize_options(self)
        self.zmq = None
        self.build_base = 'build'
        self.embed = False

    # DON'T REMOVE: distutils demands these be here even if they do nothing.
    def finalize_options(self):
        build_ext.finalize_options(self)
        self.tempdir = pjoin(self.build_temp, 'scratch')
        self.has_run = False

    def create_tempdir(self):
        self.erase_tempdir()
        os.makedirs(self.tempdir)
        if sys.platform.startswith('win'):
            # fetch libzmq.dll into local dir
            local_dll = pjoin(self.tempdir, libzmq_name + '.dll')
            if not self.config['zmq_prefix'] and not os.path.exists(local_dll):
                fatal("ZMQ directory must be specified on Windows via setup.cfg"
                      " or 'python setup.py configure --zmq=/path/to/zeromq2'")

            try:
                shutil.copy(pjoin(self.config['zmq_prefix'], 'lib', libzmq_name + '.dll'), local_dll)
            except Exception:
                if not os.path.exists(local_dll):
                    warn("Could not copy " + libzmq_name + " into zmq/, which is usually necessary on Windows."
                                                           "Please specify zmq prefix via configure --zmq=/path/to/zmq or copy "
                         + libzmq_name + " into zmq/ manually.")

    def erase_tempdir(self):
        try:
            shutil.rmtree(self.tempdir)
        except Exception:
            pass

    @property
    def compiler_type(self):
        compiler = self.compiler
        if compiler is None:
            return get_default_compiler()
        elif isinstance(compiler, str):
            return compiler
        else:
            return compiler.compiler_type

    @property
    def cross_compiling(self):
        return self.config['bdist_egg'].get('plat-name', sys.platform) != sys.platform

    def bundle_libzmq_extension(self):
        bundledir = "bundled"
        ext_modules = self.distribution.ext_modules
        if ext_modules and any(m.name == 'czmq.libzmq' for m in ext_modules):
            # I've already been run
            return

        line()
        info("Using bundled libzmq")

        # fetch sources for libzmq extension:
        if not os.path.exists(bundledir):
            os.makedirs(bundledir)

        fetch_libzmq(bundledir)

        stage_platform_header(pjoin(bundledir, 'zeromq'))

        sources = [pjoin('buildutils', 'zmq', 'initlibzmq.c')]
        sources += glob(pjoin(bundledir, 'zeromq', 'src', '*.cpp'))

        includes = [
            pjoin(bundledir, 'zeromq', 'include')
        ]

        if bundled_version < (4, 2, 0):
            tweetnacl = pjoin(bundledir, 'zeromq', 'tweetnacl')
            tweetnacl_sources = glob(pjoin(tweetnacl, 'src', '*.c'))

            randombytes = pjoin(tweetnacl, 'contrib', 'randombytes')
            if sys.platform.startswith('win'):
                tweetnacl_sources.append(pjoin(randombytes, 'winrandom.c'))
            else:
                tweetnacl_sources.append(pjoin(randombytes, 'devurandom.c'))

            sources += tweetnacl_sources
            includes.append(pjoin(tweetnacl, 'src'))
            includes.append(randombytes)
        else:
            # >= 4.2
            sources += glob(pjoin(bundledir, 'zeromq', 'src', 'tweetnacl.c'))

        libzmq = Extension(
            'czmq.libzmq',
            sources=sources,
            include_dirs=includes,
        )

        # http://stackoverflow.com/a/32765319/7205341
        if sys.platform == 'darwin':
            from distutils import sysconfig
            vars = sysconfig.get_config_vars()
            vars['LDSHARED'] = vars['LDSHARED'].replace('-bundle', '-dynamiclib')

        # register the extension:
        self.distribution.ext_modules.insert(0, libzmq)

        # use tweetnacl to provide CURVE support
        libzmq.define_macros.append(('ZMQ_HAVE_CURVE', 1))
        libzmq.define_macros.append(('ZMQ_USE_TWEETNACL', 1))

        # select polling subsystem based on platform
        if sys.platform == 'darwin' or 'bsd' in sys.platform:
            libzmq.define_macros.append(('ZMQ_USE_KQUEUE', 1))
        elif 'linux' in sys.platform:
            libzmq.define_macros.append(('ZMQ_USE_EPOLL', 1))
        elif sys.platform.startswith('win'):
            libzmq.define_macros.append(('ZMQ_USE_SELECT', 1))
        else:
            # this may not be sufficiently precise
            libzmq.define_macros.append(('ZMQ_USE_POLL', 1))

        if sys.platform.startswith('win'):
            # include defines from zeromq msvc project:
            libzmq.define_macros.append(('FD_SETSIZE', 16384))
            libzmq.define_macros.append(('DLL_EXPORT', 1))
            libzmq.define_macros.append(('_CRT_SECURE_NO_WARNINGS', 1))

            # When compiling the C++ code inside of libzmq itself, we want to
            # avoid "warning C4530: C++ exception handler used, but unwind
            # semantics are not enabled. Specify /EHsc".
            if self.compiler_type == 'msvc':
                libzmq.extra_compile_args.append('/EHsc')
            elif self.compiler_type == 'mingw32':
                libzmq.define_macros.append(('ZMQ_HAVE_MINGW32', 1))

            # And things like sockets come from libraries that must be named.
            libzmq.libraries.extend(['rpcrt4', 'ws2_32', 'advapi32'])

            # bundle MSCVP redist
            if self.config['bundle_msvcp']:
                cc = new_compiler(compiler=self.compiler_type)
                cc.initialize()
                # get vc_redist location via private API
                try:
                    cc._vcruntime_redist
                except AttributeError:
                    # fatal error if env set, warn otherwise
                    msg = fatal if os.environ.get("PYZMQ_BUNDLE_CRT") else warn
                    msg("Failed to get cc._vcruntime via private API, not bundling CRT")
                if cc._vcruntime_redist:
                    redist_dir, dll = os.path.split(cc._vcruntime_redist)
                    to_bundle = [
                        pjoin(redist_dir, dll.replace('vcruntime', name))
                        for name in ('msvcp', 'concrt')
                        ]
                    for src in to_bundle:
                        dest = localpath('zmq', basename(src))
                        info("Copying %s -> %s" % (src, dest))
                        # copyfile to avoid permission issues
                        shutil.copyfile(src, dest)

        else:
            libzmq.include_dirs.append(bundledir)

            # check if we need to link against Realtime Extensions library
            cc = new_compiler(compiler=self.compiler_type)
            cc.output_dir = self.build_temp
            libzmq.libraries.append("stdc++")
            if not sys.platform.startswith(('darwin', 'freebsd')):
                line()
                info("checking for timer_create")
                if not cc.has_function('timer_create'):
                    info("no timer_create, linking librt")
                    libzmq.libraries.append('rt')
                else:
                    info("ok")

                if pypy:
                    # seem to need explicit libstdc++ on linux + pypy
                    # not sure why
                    libzmq.libraries.append("stdc++")

    def run(self):
        self.bundle_libzmq_extension()



import os
import sys

PYVERSION = 2
if sys.version_info > (3,):
    PYVERSION = 3


ZYRE_GROUP = os.environ.get('ZYRE_GROUP', 'ZYRE')
SERVICE_PORT = os.environ.get('ZYRE_SERVICE_PORT', '49155')

GOSSIP_PORT = os.environ.get('ZYRE_SERVICE_PORT', '49154')
GOSSIP_CONNECT = os.getenv('ZYRE_GOSSIP_CONNECT')
ENDPOINT = os.getenv('ZYRE_ENDPOINT')

LOG_FORMAT = '%(asctime)s - %(levelname)s - %(name)s[%(lineno)s] - %(message)s'



import sys
import zmq

topicfilter = sys.argv[1]

# Socket to talk to server
context = zmq.Context()
socket = context.socket(zmq.SUB)
socket.connect ("tcp://localhost:%s" % 5001)

socket.setsockopt(zmq.SUBSCRIBE, topicfilter)

control = context.socket(zmq.PUSH)
control.connect("tcp://localhost:%s" % 5002)
control.send_multipart(["SUB", topicfilter]) #needs to be done periodically

while True:
    topic, messagedata = socket.recv_multipart()
    print topic, messagedata



import logging
from zmq.eventloop import ioloop
from argparse import ArgumentParser
from pyzyre.client import Client
from pyzyre._client_task import task
import zmq
from pyzyre.constants import ZYRE_GROUP, LOG_FORMAT, SERVICE_PORT, PYVERSION
import os
import select
import sys
from pprint import pprint
from pyzyre import color

logger = logging.getLogger('')


def main():
    p = ArgumentParser()
    p.add_argument('-d', '--debug', help='enable debugging', action='store_true')

    p.add_argument('-i', '--interface', help='specify zsys_interface for beacon')
    p.add_argument('--group', default=ZYRE_GROUP)
    p.add_argument('--address', default='ipc:///tmp/zyre.ipc')

    args = p.parse_args()

    loglevel = logging.INFO
    verbose = False
    if args.debug:
        loglevel = logging.DEBUG
        verbose = '1'

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)
    logging.propagate = False

    if select.select([sys.stdin, ], [], [], 0.0)[0]:
        ctx = zmq.Context()
        s = ctx.socket(zmq.PUSH)
        s.setsockopt(zmq.LINGER, 0)
        s.connect(args.address)

        content = sys.stdin.read().strip('\n')

        if PYVERSION == 2:
            content = unicode(content, encoding='utf-8', errors='ignore')

        logger.info('sending..')
        s.send_multipart([content.encode('utf-8')])
        logger.info('sent...')
        s.close()
    else:
        logger.info('running proxy..')
        ioloop.install()
        loop = ioloop.IOLoop.instance()

        zyre = Client(
            group=args.group,
            loop=loop,
            verbose=verbose,
            interface=args.interface,
            task=task
        )

        def handle_message(s, e):
            m = s.recv_multipart()

            logger.debug(m)

            zyre.shout(args.group, m[0])

        zyre.start_zyre()

        ctx = zmq.Context()
        s = ctx.socket(zmq.PULL)
        s.bind(args.address)

        loop.add_handler(s, handle_message, zmq.POLLIN)

        try:
            loop.start()
        except KeyboardInterrupt:
            logger.info('SIGINT Received')
        except Exception as e:
            logger.error(e)

        zyre.stop_zyre()

    logger.info('done')

if __name__ == '__main__':
    main()



from zyre import Zyre, ZyreEvent
import zmq
import logging
from czmq import Zsock, string_at, Zmsg
import names
import uuid
from pyzyre.constants import ZYRE_GROUP
import os
from pyzyre import color

logger = logging.getLogger('')

EVASIVE_TIMEOUT = os.environ.get('ZYRE_EVASIVE_TIMEOUT', 5000)  # zyre defaults
EXPIRED_TIMEOUT = os.environ.get('ZYRE_EXPIRED_TIMEOUT', 30000)
TRACE_EVASIVE = os.getenv('ZYRE_EVASITVE_TRACE')
NODE_NAME = os.getenv('ZYRE_NODE_NAME')
TRACE = os.getenv('ZYRE_TRACE')


def task(pipe, arg):
    args = string_at(arg)
    args = dict(item.split("=") for item in args.split(","))

    name = args.get('name')
    if not name:
        raise RuntimeError('missing node name')

    group = args.get('group', ZYRE_GROUP)

    logger.info('setting up node: %s' % name)
    n = Zyre(name)

    logger.debug('setting evasive timeout: {}'.format(EVASIVE_TIMEOUT))
    n.set_evasive_timeout(int(EVASIVE_TIMEOUT))

    logger.debug('setting experation timer: {}'.format(EXPIRED_TIMEOUT))
    n.set_expired_timeout(int(EXPIRED_TIMEOUT))

    if TRACE:
        logger.info('setting verbose...')
        logger.setLevel(logging.DEBUG)
        n.set_verbose()

    if args.get('endpoint'):
        logger.info('setting endpoint: {}'.format(args['endpoint']))
        n.set_endpoint(args['endpoint'])

    if not args.get('beacon'):
        logger.info('setting up gossip')
        if args.get('gossip_bind'):
            logger.info('binding gossip: {}'.format(args['gossip_bind']))
            n.gossip_bind(args['gossip_bind'])
        else:
            logger.info('connecting to gossip group: {}'.format(args['gossip_connect']))
            n.gossip_connect(args['gossip_connect'])

    poller = zmq.Poller()

    pipe_zsock_s = Zsock(pipe, False)
    assert pipe_zsock_s

    pipe_s = zmq.Socket(shadow=pipe_zsock_s.resolve(pipe_zsock_s).value)
    assert pipe_s

    poller.register(pipe_s, zmq.POLLIN)

    # getting the zyre [czmq] socket
    # https://marc.info/?l=zeromq-dev&m=144719752222833&w=2
    s = n.socket()
    assert s

    # re-casting as native pyzmq socket
    ss = zmq.Socket(shadow=s.resolve(s).value)
    assert ss

    # registering
    poller.register(ss, zmq.POLLIN)

    logger.info('staring node...')
    n.start()

    group = group.split('|')
    for g in group:
        logger.info('joining: %s' % g)
        n.join(g)

    pipe_zsock_s.signal(0)  # OK

    peers = {}
    terminated = False
    while not terminated:
        items = dict(poller.poll())

        try:
            if pipe_s in items and items[pipe_s] == zmq.POLLIN:
                message = Zmsg.recv(pipe)

                if not message:
                    break  # SIGINT

                msg_type = message.popstr().decode('utf-8').upper()

                # message to quit
                if msg_type == "$$STOP":
                    for g in group:
                        n.leave(g)
                    terminated = True

                elif msg_type == '$$ID':
                    pipe_s.send_string(n.uuid().decode('utf-8'))

                elif msg_type == 'WHISPER':
                    address = message.popstr().decode('utf-8')
                    msg = message.popstr().decode('utf-8')
                    m = Zmsg()
                    m.addstr(msg)
                    address = peers[str(address)]
                    address = uuid.UUID(address).hex.upper()
                    n.whisper(address, m)

                elif msg_type == 'JOIN':
                    group = message.popstr().decode('utf-8')
                    logger.debug('joining %s' % group)
                    n.join(group)

                elif msg_type == 'SHOUT':
                    g = message.popstr()
                    msg = message.popstr()
                    logger.debug('shouting[%s]: %s' % (g, msg))
                    n.shouts(g, "%s", msg)

                else:
                    logger.warn('unknown message type: {}'.format(msg_type))

            elif ss in items and items[ss] == zmq.POLLIN:
                e = ZyreEvent(n)

                msg_type = e.type().decode('utf-8').upper()
                #logger.debug('found ZyreEvent: %s' % msg_type)
                #logger.debug(e.get_msg().popstr())

                if msg_type == "ENTER":
                    logger.debug('ENTER {} - {}'.format(e.peer_name(), e.peer_uuid()))
                    peers[e.peer_name()] = e.peer_uuid()
                    pipe_s.send_multipart(['ENTER', e.peer_uuid(), e.peer_name()])
                    #headers = e.headers() # zlist

                elif msg_type == 'LEAVE':
                    logger.debug('LEAVE [{}] [{}]'.format(e.group(), e.peer_name()))
                    pipe_s.send_multipart(['LEAVE', e.peer_name(), e.group()])

                elif msg_type == 'JOIN':
                    logger.debug('JOIN [{}] [{}]'.format(e.group(), e.peer_name()))
                    pipe_s.send_multipart(['JOIN', e.peer_name(), e.group()])

                elif msg_type == "SHOUT":
                    m = e.get_msg().popstr()
                    logger.debug('SHOUT [{}] [{}]: {} - {}'.format(e.group(), e.peer_name(), e.peer_uuid(), m))
                    pipe_s.send_multipart(['SHOUT', e.group(), e.peer_name(), e.peer_uuid(), m])

                elif msg_type == "WHISPER":
                    m = e.get_msg().popstr()
                    logger.debug('WHISPER [{}]: {}'.format(e.peer_name(), m))
                    pipe_s.send_multipart(['WHISPER', e.peer_name(), e.peer_uuid(), m])

                elif msg_type == 'EXIT':
                    logger.debug('EXIT [{}] [{}]'.format(e.group(), e.peer_name()))
                    if e.peer_name() in peers:
                        del peers[e.peer_name()]
                    pipe_s.send_multipart(['EXIT', e.peer_name(), str(len(peers))])

                elif msg_type == 'EVASIVE':
                    if TRACE or TRACE_EVASIVE:
                        logger.debug('EVASIVE {}'.format(e.peer_name()))

                else:
                    logger.warn('unknown message type: {}'.format(msg_type))
        except Exception as e:
            logger.exception("Unhandled exception in main io loop")

    logger.info('shutting down node')
    n.stop()

    logger.debug('done')


import netifaces as ni
import netaddr
import os.path
import socket
from pprint import pprint


def resolve_interface(address):
    address = netaddr.IPAddress(address)

    for i in ni.interfaces():
        ii = ni.ifaddresses(i)
        if not ii.get(ni.AF_INET):
            continue

        ii = ii[ni.AF_INET]
        ii = netaddr.IPNetwork('{}/{}'.format(ii[0]['addr'], ii[0]['netmask']))
        try:
            if address in ii:
                return i
        except AttributeError:
            pass


def resolve_address(i):
    return interface_to_address(i)


def interface_to_address(i):
    i = ni.ifaddresses(i)
    i = i[ni.AF_INET]
    i = i[0]['addr']
    return i


def default_address():
    i = default_interface()
    return interface_to_address(i)


def default_interface():
    try:
        return ni.gateways()['default'][ni.gateways()['default'].keys()[0]][1]
    except IndexError:
        raise RuntimeError("Unable to determine endpoint address")


def resolve_endpoint(port, address=None, interface=None):
    endpoint = address

    if address:
        if address.startswith(('ipc://', 'tcp://', 'udp://')):
            return address

        try:
            # is it an ip address?
            socket.inet_aton(address)
        except socket.error:
            try:
                # is it an FQDN?
                socket.gethostbyname(address)
            except socket.error:
                if os.path.basename(address):
                    return 'ipc://{}'.format(address)

        if 'tcp://' not in address:
            endpoint = 'tcp://%s' % address

        if port not in address:
            endpoint = '{}:{}'.format(endpoint, port)

        return endpoint

    if interface and interface != '*':
        i = interface_to_address(interface)
        endpoint = 'tcp://{}:{}'.format(i, port)
        return endpoint

    endpoint = default_address()
    endpoint = 'tcp://{}:{}'.format(endpoint, port)

    return endpoint


def resolve_gossip(port, address=None):
    if not address:
        address = resolve_endpoint(port)
    elif len(address) <= 5:
        # they sent us an interface...
        address = resolve_endpoint(port, interface=address)
    else:
        address = resolve_endpoint(port, address)

    return address



import sys
import pytest
from zmq.eventloop import ioloop
from pyzyre.client import Client
from pyzyre._client_task import task
import netifaces as ni

from time import sleep
from pprint import pprint
import zmq

ioloop.install()

@pytest.fixture
def iface():
    if sys.platform == 'darwin':
        return 'en0'
    else:
        if 'eth0' in ni.interfaces():
            return 'eth0'
        else:
            return ni.interfaces()[1]


def test_client_beacon(iface):
    # test with multi-process framework..
    c1 = Client(task=task, verbose='1')
    c1.start_zyre()

    c2 = Client(task=task, verbose='1')
    c2.start_zyre()

    sleep(0.01)

    loop = ioloop.IOLoop.instance()
    loop.add_handler(c1.actor, c1.handle_message, zmq.POLLIN)

    def test_fcn():
        c2.shout('ZYRE', 'TEST')
        sleep(1)

    loop.run_sync(test_fcn)

    c1.stop_zyre()
    c2.stop_zyre()

    loop.remove_handler(c1.actor)

    # cleanup
    sleep(1)


def test_client_gossip(iface):
    # test with multi-process framework..
    c1 = Client(task=task, verbose='1', gossip_bind='ipc:///tmp/gossip.ipc', endpoint='ipc:///tmp/c1.ipc')
    c1.start_zyre()

    c2 = Client(task=task, verbose='1', gossip_connect='ipc:///tmp/gossip.ipc', endpoint='ipc:///tmp/c2.ipc')
    c2.start_zyre()

    sleep(0.01)

    loop = ioloop.IOLoop.instance()
    loop.add_handler(c1.actor, c1.handle_message, zmq.POLLIN)

    def test_fcn():
        c2.shout('ZYRE', 'TEST')
        sleep(1)

    loop.run_sync(test_fcn)

    c1.stop_zyre()
    c2.stop_zyre()

    loop.remove_handler(c1.actor)

    # cleanup
    sleep(1)


#!/usr/bin/env python

from __future__ import with_statement, print_function

import os
import sys
from distutils.version import LooseVersion
from setuptools import setup
import setuptools.command.build_py
from setuptools.command.develop import develop
from buildutils.zmq.configure import Configure as ConfigureZmq
from buildutils.czmq.configure import Configure as ConfigureCzmq, ConfigureSDist as ConfigureCzmqSdist
from buildutils.zyre.configure import Configure as ConfigureZyre, ConfigureSDist as ConfigureZyreSdist
from distutils.command.sdist import sdist
from distutils.command.install import install
from ctypes import *
from buildutils.czmq.msg import fatal
from subprocess import Popen, PIPE
from setuptools.dist import Distribution
from pprint import pprint

import versioneer

# vagrant doesn't appreciate hard-linking
if os.environ.get('USER') == 'vagrant' or os.path.isdir('/vagrant'):
    del os.link

# https://www.pydanny.com/python-dot-py-tricks.html
if sys.argv[-1] == 'test':
    test_requirements = [
        'pytest',
    ]
    try:
        modules = map(__import__, test_requirements)
    except ImportError as e:
        err_msg = e.message.replace("No module named ", "")
        msg = "%s is not installed. Install your test requirements." % err_msg
        raise ImportError(msg)
    r = os.system('py.test test -v -s')
    if r == 0:
        sys.exit()
    else:
        raise RuntimeError('tests failed')

try:
    import Cython
    if LooseVersion(Cython.__version__) < LooseVersion('0.16'):
        raise ImportError("Cython >= 0.16 required, found %s" % Cython.__version__)
    try:
        # Cython 0.25 or later
        from Cython.Distutils.old_build_ext import old_build_ext as build_ext_c
    except ImportError:
        from Cython.Distutils import build_ext as build_ext_c
except Exception as e:
    raise ImportError('Cython >= 0.16 required')

libuuid = 'libuuid.so'
if sys.platform == 'darwin':
    libuuid = None

if sys.platform.startswith('win'):
    libuuid = None

if libuuid:
    try:
        cdll.LoadLibrary(libuuid)
    except OSError:
        print("\nuuid.so needs to be installed, for more info checkout:\nhttps://github.com/wesyoung/pyzyre/wiki\n")
        raise SystemExit

zmqlib = 'libzyre.so'
if sys.platform == 'darwin':
    zmqlib = 'libzyre.dylib'


pypy = 'PyPy' in sys.version


# set dylib ext:
if sys.platform.startswith('win'):
    lib_ext = '.dll'
else:
    lib_ext = '.so'

for idx, arg in enumerate(list(sys.argv)):
    if arg == 'egg_info':
        sys.argv.pop(idx)
        sys.argv.insert(idx, 'build')
        sys.argv.insert(idx+1, 'egg_info')


class zbuild_ext(build_ext_c):

    def finalize_options(self):
        build_ext_c.finalize_options(self)
        # set binding so that compiled methods can be inspected
        self.cython_directives['binding'] = True

    def build_extensions(self):
        # if self.compiler.compiler_type == 'mingw32':
        #     customize_mingw(self.compiler)
        return build_ext_c.build_extensions(self)

    def build_extension(self, ext):
        build_ext_c.build_extension(self, ext)

    def run(self):

        if 'develop' in sys.argv:
            return

        self.distribution.run_command('configure_zmq')
        self.distribution.run_command('configure_czmq')
        self.distribution.run_command('configure')

        r = build_ext_c.run(self)

        if sys.platform != 'darwin':
            return r

        cmds = []
        cmds.append(
            ['install_name_tool', '-change', 'build/lib/czmq/libzmq.so', 'czmq/libzmq.so',
             'build/lib/czmq/libczmq.so']
        )
        cmds.append(
            ['install_name_tool', '-change', 'build/lib/czmq/libzmq.so', 'czmq/libzmq.so',
             'build/lib/zyre/libzyre.so']
        )

        cmds.append(
            ['install_name_tool', '-change', 'build/lib/czmq/libczmq.so', 'czmq/libczmq.so',
             'build/lib/zyre/libzyre.so']
        )

        for c in cmds:
            try:
                p = Popen(c, stdout=PIPE, stderr=PIPE)

            except OSError:
                fatal("install_name_tool not found, cannot patch libzmq for bundling.")

            out, err = p.communicate()
            if p.returncode:
                fatal("Could not patch bundled libzmq install_name: %s" % err, p.returncode)

        return r


class CheckSDist(sdist):
    def run(self):
        self.run_command('configure_sdist_czmq')
        self.run_command('configure_sdist_zyre')

        sdist.run(self)


class BinaryDistribution(Distribution):
    """Distribution which always forces a binary package with platform name"""
    def has_ext_modules(foo):
        if sys.platform == 'darwin':
            return True


class BuildPyCommand(setuptools.command.build_py.build_py):
  """Custom build command."""

  def run(self):

    if 'bdist_wheel' in sys.argv and os.getenv('PYZYRE_COMPILE') == '1':
        self.run_command('build_ext')
    else:
        # we're probably trying to install
        # check to make sure libzyre.so exists
        try:
            cdll.LoadLibrary(zmqlib)
        except OSError:
            e = "\nlibzyre.so needs to be installed, for more info checkout:\nhttps://github.com/wesyoung/pyzyre/wiki\n"
            print(e)
            raise SystemError(e)

    setuptools.command.build_py.build_py.run(self)


class DevelopCommand(develop):
    def run(self):
        try:
            cdll.LoadLibrary(zmqlib)
        except OSError:
            print(
                "\nlibzyre.so needs to be installed, for more info checkout:\nhttps://github.com/wesyoung/pyzyre/wiki\n"
            )
            raise SystemExit

        self.run_command('sdist')
        develop.run(self)


cmdclass = versioneer.get_cmdclass()
cmdclass = {
    'configure': ConfigureZyre,
    'configure_zmq': ConfigureZmq,
    'configure_czmq': ConfigureCzmq,
    'build_ext': zbuild_ext,
    'build_py': BuildPyCommand,
    'develop': DevelopCommand,
    'sdist': CheckSDist,
    'configure_sdist_czmq': ConfigureCzmqSdist,
    'configure_sdist_zyre': ConfigureZyreSdist,
}


packages = ['czmq', 'zyre', 'pyzyre']

package_data = {
    'zyre': ['*' + lib_ext]
}

extensions = []


setup(
    name="pyzyre",
    version=versioneer.get_version(),
    packages=packages,
    ext_modules=extensions,
    package_data=package_data,
    author="Wes Young",
    author_email="wes@barely3am.com",
    url='https://github.com/wesyoung/pyzyre',
    description="higher level bundled pythonic bindings for Zyre",
    long_description="",
    license="MPL2",
    cmdclass=cmdclass,
    distclass=BinaryDistribution,
    install_requires=[
        'netifaces',
        'netaddr',
        'cython>=0.16',
        'names',
        'tornado',
        'pyzmq>=16.0.1'
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)'
    ],
    entry_points={
        'console_scripts': [
            'zyre-chat=pyzyre.client:main',
            'zyre-proxy=pyzyre.proxy:main',
            'zyre-gateway=pyzyre.gateway:main',
            'zyre-broker=pyzyre.broker:main'
        ]
    }
)




from argparse import ArgumentParser
import zmq
import logging
from pyzyre.utils import resolve_endpoint
from zmq.eventloop import ioloop
from pyzyre.constants import SERVICE_PORT, ZYRE_GROUP, LOG_FORMAT, GOSSIP_PORT
from pyzyre.client import Client, DefaultHandler


logger = logging.getLogger(__name__)


class BrokerHandler(DefaultHandler):
    def on_shout(self, client, group, peer, address, message):
        logger.info('SHOUT[{}][{}]: {}'.format(group, peer, message))


# see examples/pub.py and sub.py
def main():
    p = ArgumentParser()

    endpoint = resolve_endpoint(SERVICE_PORT)
    gossip = resolve_endpoint(GOSSIP_PORT)

    p.add_argument('-g','--gossip-bind', help='bind gossip endpoint on this node [default %(default)s]', default=gossip)
    p.add_argument('-e', '--endpoint', help='specify ip listening endpoint [default %(default)s]', default=endpoint)
    p.add_argument('-d', '--debug', help='enable debugging', action='store_true')

    p.add_argument('--group', help="group to join [default %(default)s]", default=ZYRE_GROUP)

    args = p.parse_args()

    loglevel = logging.INFO
    if args.debug:
        loglevel = logging.DEBUG

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)
    logging.propagate = False

    ioloop.install()
    loop = ioloop.IOLoop.instance()

    client = Client(
        handler=BrokerHandler(),
        group=args.group,
        loop=loop,
        gossip_bind=args.gossip_bind,
        endpoint=args.endpoint,
    )

    client.start_zyre()
    loop.add_handler(client.actor, client.handle_message, zmq.POLLIN)

    terminated = False
    while not terminated:
        try:
            logger.info('starting loop...')
            loop.start()

        except KeyboardInterrupt:
            logger.info('SIGINT Received')
            terminated = True

        except Exception as e:
            terminated = True
            logger.error(e)

    logger.info('shutting down..')

    client.stop_zyre()

if __name__ == '__main__':
    main()
